#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>
#include <memory>
#if defined(LINUX)
# include <netdb.h>
# include <sys/socket.h>
# include <fcntl.h>
# include <netinet/in.h>
# include <sys/time.h>
# include <arpa/inet.h>
# include <unistd.h>
#endif

#include "Aw.h"
#include "AwV4.h"

/*---------------------------------------------------------------------------*/

/* Specify a universe host here */
#define UNIVERSE_HOST "auth.activeworlds.com"

/* Specify a universe port here */
#define UNIVERSE_PORT (6670)

/* Specify what world to test in */
#define LOGIN_WORLD "myworld"

/* Specify a citnum to login with */
#define LOGIN_CITNUM (123456)

/* Specify the above citnum's privilege password */
#define LOGIN_PPW "mypass"

/*---------------------------------------------------------------------------*/

#if defined(_MSC_VER)
# undef AW_STATIC
# pragma comment(lib, "aw.lib")
#endif

void exit_aw_error(char* method, int rc)
{
	printf("%s failed: reason %d\n", method, rc);
	exit(rc);
}

template<typename SDK_T, class CLASS_T>
void do_build(CLASS_T& obj)
{
	SDK_T* data = obj.GetObjectData();
	AwV4::PrepareToBuild(data);

	aw_string_set(AW_OBJECT_MODEL, "rcube.rwx");
	aw_int_set(AW_OBJECT_X, -9000);
	aw_int_set(AW_OBJECT_Y, 0);
	aw_int_set(AW_OBJECT_Z, -9000);
	aw_int_set(AW_OBJECT_OWNER, LOGIN_CITNUM);
	aw_object_add();

	free(data);
}

void zone_test()
{
	AwV4::Zone zone;

	AwV4::SimpleString& camtag = zone.CameraTag();
	camtag = "rcube";

	unsigned short& flg = zone.Flags();
	flg |= AW_ZONE_FLAG_VISIBLE;

	do_build<aw_object_data_zone, AwV4::Zone>(zone);
}

void particle_test()
{
	AwV4::ParticleEmitter emitter;

	// this particle is dr. squailboont's infamous random fountain

	AwV4::SimpleString& assets = emitter.AssetList();
	assets = "c_flare1,c_flare2,c_nebula,c_portal,c_soft1,c_spotlight,c_spotlight2,c_spotlight3,c_star,c_star2,c_sun,c_sun2";

	emitter.ColorStart() = 0xFFFFFF;
	emitter.ReleaseMin() = 20;
	emitter.ReleaseMax() = 90;
	emitter.ReleaseCount() = 6;
	emitter.Lifespan() = 8000;
	emitter.FadeIn() = 500;
	emitter.FadeOut() = 7000;
	emitter.RenderStyle() = AW_PARTICLE_DRAW_BRIGHT;

	aw_type_vector_range& speed = emitter.Speed();
	speed.min.x = -3.0f;
	speed.min.y =  5.0f;
	speed.min.z = -3.0f;
	speed.max.x =  3.0f;
	speed.max.y =  5.0f;
	speed.max.z =  3.0f;

	aw_type_vector_range& accel = emitter.Accel();
	accel.min.y = -2.5f;
	accel.max.y = -2.5f;

	aw_type_vector_range& size = emitter.Size();
	size.min.x = 0.2f;
	size.min.y = 0.2f;
	size.min.z = 2.0f;
	size.max.x = 1.6f;
	size.max.y = 1.6f;
	size.max.z = 2.0f;

	do_build<aw_object_data_particles, AwV4::ParticleEmitter>(emitter);
}

void mover_test()
{
	AwV4::Mover mov;

	mov.Type() = AW_MOVER_TYPE_PASSIVE;
	mov.Name() = "lawlz";
	mov.BumpName() = "omg";
	mov.AddWaypoint(new AwV4::Mover::Waypoint(0, 10, 0, 0, 0, 0, static_cast<unsigned short>(2.0f * 100), 255));

	do_build<aw_object_data_mover, AwV4::Mover>(mov);
}

void do_v4_test()
{
//	zone_test();
//	particle_test();
	mover_test();
}

int main(int, char**)
{
	aw_init(AW_BUILD);

	int rc = 0;

	if ((rc = aw_init(AW_BUILD)))
		exit_aw_error("aw_init", rc);

	if ((rc = aw_create(UNIVERSE_HOST, UNIVERSE_PORT, 0)))
		exit_aw_error("aw_create", rc);

	aw_int_set(AW_LOGIN_OWNER, LOGIN_CITNUM);
	aw_string_set(AW_LOGIN_PRIVILEGE_PASSWORD, LOGIN_PPW);
	aw_string_set(AW_LOGIN_NAME, "awv4test");
	aw_string_set(AW_LOGIN_APPLICATION, "AwV4 Library Test Program");

	if ((rc = aw_login()))
		exit_aw_error("aw_login", rc);

	aw_bool_set(AW_ENTER_GLOBAL, true);

	if ((rc = aw_enter(LOGIN_WORLD)))
		exit_aw_error("aw_enter", rc);

	aw_state_change();
	aw_wait(500);

	aw_say("Barnacles!");
	do_v4_test();

	aw_term();

	return 0;
}

/*---------------------------------------------------------------------------*/
